#!/bin/sh
lupdate tools/tools.pro
lupdate actiontools/actiontools.pro
lupdate executer/executer.pro
lupdate actexecuter/actexecuter.pro
lupdate gui/gui.pro
lupdate actions/actionpackinternal/actionpackinternal.pro
lupdate actions/actionpackwindows/actionpackwindows.pro
lupdate actions/actionpackdevice/actionpackdevice.pro
lupdate actions/actionpacksystem/actionpacksystem.pro
lupdate actions/actionpackdata/actionpackdata.pro
