/*
    Actiona
    Copyright (C) 2008-2015 Jonathan Mercier-Ganady

    Actiona is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Actiona is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

    Contact : jmgr@jmgr.info
*/

#ifndef LANGUAGES_H
#define LANGUAGES_H

#include "tools_global.h"

#include <QPair>
#include <QStringList>

namespace Tools
{
    extern TOOLSSHARED_EXPORT const QPair<QStringList, QStringList> languagesName;

    QString TOOLSSHARED_EXPORT locale();
    void TOOLSSHARED_EXPORT installQtTranslator(const QString &locale);
    void TOOLSSHARED_EXPORT installTranslator(const QString &componentName, const QString &locale);
    int TOOLSSHARED_EXPORT languageNameToIndex(const QString &languageName);
}

#endif // LANGUAGES_H
