/*
	Actiona
	Copyright (C) 2008-2015 Jonathan Mercier-Ganady

	Actiona is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Actiona is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.

	Contact : jmgr@jmgr.info
*/

#ifndef READTEXTFILEINSTANCE_H
#define READTEXTFILEINSTANCE_H

#include "actioninstance.h"
#include "stringlistpair.h"

namespace Actions
{
	class ReadTextFileInstance : public ActionTools::ActionInstance
	{
		Q_OBJECT
		Q_ENUMS(Mode)

	public:
		enum Mode
		{
			Full,
			Selection
		};
		enum Exceptions
		{
			CannotOpenFileException = ActionTools::ActionException::UserException
		};

		ReadTextFileInstance(const ActionTools::ActionDefinition *definition, QObject *parent = 0)
			: ActionTools::ActionInstance(definition, parent)												{}

		static ActionTools::StringListPair modes;

		void startExecution();

	private:
		Q_DISABLE_COPY(ReadTextFileInstance)
	};
}

#endif // READTEXTFILEINSTANCE_H
